@@ -6,7 +6,31 @@
 ##########################
 
 
-# Adjusted Rand Index
+#' Adjusted Rand Index
+#' 
+#' This function calculates the adjusted Rand index between two
+#' clusterings/partitions.
+#' 
+#' The \code{\link{adj.rand.index}} function takes as its input two
+#' clusterings/partitions in cluster label notation and computes the adjusted
+#' Rand index. The adjusted Rand index is at most \code{1.0} and large numbers
+#' indicate high similarity.
+#' 
+#' @param c1 A vector containing cluster labels for a clustering/partition.
+#' @param c2 A vector containing cluster labels for a clustering/partition.
+#' @return A numeric vector of length one representing the adjusted Rand index
+#' between the two clusterings/partitions.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @seealso \code{\link{estimate.partition}}
+#' @references Hubert, L. and Arabie, P. (1985), Comparing partitions,
+#' \emph{Journal of Classification}, \bold{2}, 193-218.
+#' @examples
+#' 
+#' truth    <- c(1,1,2,2,2,1,3,3,3)
+#' estimate <- c(1,2,2,2,2,1,2,3,3)
+#' adj.rand.index(truth,estimate)
+#' 
+#' @export adj.rand.index
 adj.rand.index <- function(c1,c2) {
   n <- length(c1)
   if ( length(c2) != n ) stop("Clusterings must be the same length.")
@@ -25,7 +49,51 @@ adj.rand.index <- function(c1,c2) {
 #### To be added: "variation of information" by Meila (2007).
 
 
-# Mass
+
+#' Mass, Discount, and Temperature Parameters
+#' 
+#' These functions set the mass, discount, and temperature parameters and, in
+#' the case of them being random, specify the parameters of their distribution.
+#' 
+#' If no parameters are specified, the mass parameter defaults to \code{1.2},
+#' the discount parameter defaults to \code{0.05}, the temperature parameter
+#' defaults to \code{3.0}. If the mass parameter is random, the default shape
+#' and rate parameters of the gamma distribution are \code{2.5} and \code{2},
+#' respectively. If the discount parameter is random, the default shape
+#' parameters of the beta distribution are \code{1.0} and \code{1.0}. If the
+#' temperature parameter is random, the default shape and rate parameters of
+#' the gamma distribution are \code{2} and \code{0.5}, respectively.
+#' 
+#' @aliases mass discount temperature print.shallot.mass print.shallot.discount
+#' print.shallot.temperature
+#' @param ... A number greater than \code{0.0} representing the value of the
+#' mass, discount, or temperaure parameters.  Or, in the case of them being
+#' random, a vector of two numbers representing either: i. the shape and rate
+#' parameters of the gamma distribution for the mass or temperature, or ii. the
+#' shape parameters of the beta distribution for the discount.  This argument
+#' is currently ignored for the associated print functions.
+#' @param x An object from the \code{\link{mass}}, \code{\link{discount}}, or
+#' \code{\link{temperature}} functions.
+#' @param fixed If \code{TRUE}, the parameter is fixed.  If \code{FALSE}, the
+#' parameter value is samples from either: i. a gamma distribution for the mass
+#' or temperature, or ii. a beta distribution for the discount.
+#' @return An object of class \code{shallot.mass}, \code{shallot.discount}, or
+#' \code{shallot.temperature}.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @examples
+#' 
+#' mass()
+#' mass(1.0)
+#' mass(1.4, fixed=FALSE)
+#' mass(0.5, 1, fixed=FALSE)
+#' discount()
+#' discount(0.2)
+#' discount(1, 3, fixed=FALSE)
+#' temperature()
+#' temperature(2)
+#' temperature(2, 4, fixed=FALSE)
+#' 
+#' @export mass
 mass <- function(...,fixed=TRUE) {
   x <- c(...)
   fixed <- as.logical(fixed)
@@ -57,6 +125,8 @@ mass <- function(...,fixed=TRUE) {
   else s$Mass.factory(mass$shape,mass$rate,s$rdg())
 }
 
+#' @rdname mass
+#' @export
 print.shallot.mass <- function(x, ...) {
   if ( x$fixed ) cat("mass fixed at ",x$value,"\n",sep="")
   else cat("mass at ",x$value," and distributed Gamma(shape=",x$shape,",rate=",x$rate,")\n",sep="")
@@ -69,7 +139,8 @@ print.shallot.mass <- function(x, ...) {
 
 
 
-# Discount
+#' @rdname mass
+#' @export
 discount <- function(...,fixed=TRUE) {
   x <- c(...)
   fixed <- as.logical(fixed)
@@ -101,6 +172,8 @@ discount <- function(...,fixed=TRUE) {
   else s$Discount.factory(discount$shape,discount$rate,s$rdg())
 }
 
+#' @rdname mass
+#' @export
 print.shallot.discount <- function(x, ...) {
   if ( x$fixed ) cat("discount fixed at ",x$value,"\n",sep="")
   else cat("discount at ",x$value," and distributed Beta(shape1=",x$shape1,",shape2=",x$shape2,")\n",sep="")
@@ -113,7 +186,37 @@ print.shallot.discount <- function(x, ...) {
 
 
 
-# Permutation
+#' Permutation
+#' 
+#' These function define a permutation for subsequent use.
+#' 
+#' A valid permutation of length \code{n} is an integer vector of length
+#' \code{n} containing each integer \code{1}, \code{2},... \code{n} only once.
+#' 
+#' @aliases permutation print.shallot.permutation
+#' @param ... For the function \code{\link{permutation}}, a permutation of the
+#' integers \code{1}, \code{2},... \code{n}, where \code{n} is the length of
+#' the vector.  For the function \code{\link{print.shallot.permutation}}, this
+#' is ignored.
+#' @param n.items An optional argument provided instead of \code{...} to
+#' request a random partition.  The argument \code{fixed} must be \code{FALSE}.
+#' @param x An object of class \code{shallot.permutation}.
+#' @param fixed Should the permutation be fixed?
+#' @return An object of class \code{shallot.permutation}.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @seealso \code{\link{attraction}}
+#' @references \href{https://doi.org/10.1080/01621459.2016.1165103}{Dahl, D. B.,
+#' Day, R., and Tsai, J. (2017), Random Partition Distribution Indexed by
+#' Pairwise Information, \emph{Journal of the American Statistical
+#' Association}, 112, 721-732. <DOI:10.1080/01621459.2016.1165103>}
+#' @examples
+#' 
+#' ## Demonstrate permutation.
+#' permutation(c(3, 1, 2, 5, 4))
+#' permutation(c(3, 1, 2, 5, 4), fixed=FALSE)
+#' permutation(n.items=5, fixed=FALSE)
+#' 
+#' @export permutation
 permutation <- function(...,n.items=NULL,fixed=TRUE) {
   x <- c(...)
   fixed <- as.logical(fixed)
@@ -147,6 +250,8 @@ permutation <- function(...,n.items=NULL,fixed=TRUE) {
   }
 }
 
+#' @rdname permutation
+#' @export
 print.shallot.permutation <- function(x, ...) {
   if ( x$fixed ) {
     cat("permutation of ",x$n.items," items fixed at\n",sep="")
@@ -163,8 +268,8 @@ print.shallot.permutation <- function(x, ...) {
 # permutation(2,3,4,1,fixed=FALSE)
 
 
-
-# Temperature
+#' @rdname mass
+#' @export
 temperature <- function(...,fixed=TRUE) {
   x <- c(...)
   fixed <- as.logical(fixed)
@@ -188,6 +293,8 @@ temperature <- function(...,fixed=TRUE) {
   result
 }
 
+#' @rdname mass
+#' @export
 print.shallot.temperature <- function(x, ...) {
   if ( x$fixed ) cat("temperature fixed at ",x$value,"\n",sep="")
   else cat("temperature at ",x$value," and distributed Gamma(shape=",x$shape,",rate=",x$rate,")\n",sep="")
@@ -199,7 +306,48 @@ print.shallot.temperature <- function(x, ...) {
 
 
 
-# Decay functions
+#' Decay Functions
+#' 
+#' These functions specify the decay to map distances to attractions.
+#' 
+#' There are currently three choices for decay functions: reciprocal,
+#' exponential, and subtraction.
+#' 
+#' The reciprocal decay maps a distance \code{d} to an attraction \code{a} as
+#' follows: \code{a = 1/d^t}, where \code{t} is the temperature.
+#' 
+#' The exponential decay maps a distance \code{d} to an attraction \code{a} as
+#' follows: \code{a = exp(-t*d)}, where \code{t} is the temperature.
+#' 
+#' The subtract decay maps a distance \code{d} to an attraction \code{a} as
+#' follows: \code{a = (m-d)^t}, where \code{t} is the temperature and \code{m}
+#' is the maximum distance in \code{distance} multiplied by the supplied
+#' \var{multiplier}.
+#' 
+#' @aliases decay decay.reciprocal decay.exponential decay.subtraction
+#' print.shallot.decay
+#' @param temperature An object of class \code{shallot.temperature}.
+#' @param distance An object of class \code{dist}.
+#' @param multiplier An scalar greater than \code{1.0} to ensure that
+#' attractions from \code{decay.subtraction} are finite.
+#' @param x An object of class \code{shallot.decay}.
+#' @param ... Currently ignored.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @seealso \code{\link[stats]{dist}}, \code{\link{temperature}},
+#' \code{\link{attraction}}
+#' @references \href{https://doi.org/10.1080/01621459.2016.1165103}{Dahl, D. B.,
+#' Day, R., and Tsai, J. (2017), Random Partition Distribution Indexed by
+#' Pairwise Information, \emph{Journal of the American Statistical
+#' Association}, 112, 721-732. <DOI:10.1080/01621459.2016.1165103>}
+#' @export
+#' @examples
+#' 
+#' temp <- temperature(1.0)
+#' distance <- dist(scale(USArrests))
+#' decay1 <- decay.reciprocal(temp,distance)
+#' decay2 <- decay.exponential(temp,distance)
+#' decay3 <- decay.subtraction(temp,distance)
+#' 
 decay.reciprocal <- function(temperature,distance) {
   if ( ! inherits(distance,"dist") ) stop("'distance' must be of class 'dist'")
   x1 <- -log(.Machine$double.xmin)/log(max(distance))
@@ -211,6 +359,8 @@ decay.reciprocal <- function(temperature,distance) {
   decay.generic(temperature,distance,type="reciprocal",max.temperature=max.temperature)
 }
 
+#' @rdname decay.reciprocal
+#' @export
 decay.exponential <- function(temperature,distance) {
   if ( ! inherits(distance,"dist") ) stop("'distance' must be of class 'dist'")
   x1 <- -log(.Machine$double.xmin)/max(distance)
@@ -222,6 +372,8 @@ decay.exponential <- function(temperature,distance) {
   decay.generic(temperature,distance,type="exponential",max.temperature=max.temperature)
 }
 
+#' @rdname decay.reciprocal
+#' @export
 decay.subtraction <- function(temperature,distance,multiplier=1.01) {
   if ( ! inherits(distance,"dist") ) stop("'distance' must be of class 'dist'")
   if ( ( length(multiplier) != 1 ) || ( ! is.numeric(multiplier) ) || ( multiplier <= 1.0 ) ) stop("'multiplier' must be a scalar greater than 1.")
@@ -268,6 +420,8 @@ decay.generic <- function(temperature,distance,type,max.temperature,max.distance
   }
 }
 
+#' @rdname decay.reciprocal
+#' @export
 print.shallot.decay <- function(x, ...) {
   cat(x$type,"decay function with ")
   if ( x$temperature$fixed ) cat("temperature fixed at ",x$temperature$value,"\n",sep="")
@@ -289,7 +443,37 @@ print.shallot.decay <- function(x, ...) {
 
 
 
-# Attraction
+#' Attraction
+#' 
+#' This function creates an attraction from a permutation and a decay in
+#' preparation for use in the \code{\link{ewens.attraction}},
+#' \code{\link{ewens.pitman.attraction}}, and \code{\link{ddcrp}} functions.
+#' For details on each of these arguments, please see the links below.
+#' 
+#' 
+#' @aliases attraction print.shallot.attraction as.matrix.shallot.attraction
+#' @param permutation An object of class \code{shallot.permutation} encoding
+#' the permutation of the items.
+#' @param decay An object of class \code{shallot.decay} detailing the
+#' transformation from distances to attractions.
+#' @param x An object of class \code{shallot.attraction}.
+#' @param ... Currently ignored.
+#' @return An object of class \code{shallot.attraction}.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @seealso \code{\link{ddcrp}}, \code{\link{decay}},
+#' \code{\link{ewens.attraction}}, \code{\link{ewens.pitman.attraction}},
+#' \code{\link{permutation}}
+#' @references \href{https://doi.org/10.1080/01621459.2016.1165103}{Dahl, D. B.,
+#' Day, R., and Tsai, J. (2017), Random Partition Distribution Indexed by
+#' Pairwise Information, \emph{Journal of the American Statistical
+#' Association}, 112, 721-732. <DOI:10.1080/01621459.2016.1165103>}
+#' @examples
+#' 
+#' permutation <- permutation(n.items=50,fixed=FALSE)
+#' decay <- decay.exponential(temperature(1.0),dist(scale(USArrests)))
+#' attraction(permutation, decay)
+#' 
+#' @export attraction
 attraction <- function(permutation, decay) {
   if ( ( is.vector(permutation) ) && ( length(permutation) == 1 ) && ( missing(decay) ) ) {
     result <- list(constant=TRUE,n.items=as.integer(permutation))
@@ -338,6 +522,8 @@ attraction <- function(permutation, decay) {
   }
 }
 
+#' @rdname attraction
+#' @export
 print.shallot.attraction <- function(x, ...) {
   cat("attraction for ",x$n.items," items\n",sep="")
   if ( ! is.null(x$distance) ) {
@@ -354,6 +540,8 @@ print.shallot.attraction <- function(x, ...) {
   }
 }
 
+#' @rdname attraction
+#' @export
 as.matrix.shallot.attraction <- function(x, ...) {
   stop("No yet implemented.")
   y <- x$ref$toArray()
@@ -371,15 +559,56 @@ as.matrix.shallot.attraction <- function(x, ...) {
 
 
 
-# Ewens
+#' Partition Distributions
+#' 
+#' These functions specify the Ewens, Ewens-Pitman, Ewens attraction,
+#' Ewens-Pitman attraction, and ddCRP distributions which would then be used in
+#' the \code{\link{sample.partitions} function.}
+#' 
+#' 
+#' @aliases partition.distribution ewens print.shallot.distribution.ewens
+#' ewens.pitman print.shallot.distribution.ewensPitman ewens.attraction
+#' print.shallot.distribution.ewensAttraction ewens.pitman.attraction
+#' print.shallot.distribution.ewensPitmanAttraction ddcrp
+#' print.shallot.distribution.ddcrp
+#' @param mass An object of class \code{shallot.mass}.
+#' @param discount An object of class \code{shallot.discount}.
+#' @param attraction An object of class \code{shallot.attraction}.
+#' @param n.items An integer containing the number of items to partition.
+#' @param names A character vector containing the names of the items.  The
+#' default names are of the form \dQuote{c1}, \dQuote{c2}, etc.
+#' @param x An object of class \code{shallot.distribution}.
+#' @param ... Currently ignored.
+#' @return An object of class \code{shallot.distribution}.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @seealso \code{\link{mass}}, \code{\link{discount}},
+#' \code{\link{attraction}}, \code{\link{sample.partitions}}
+#' @references \href{https://doi.org/10.1080/01621459.2016.1165103}{Dahl, D. B.,
+#' Day, R., and Tsai, J. (2017), Random Partition Distribution Indexed by
+#' Pairwise Information, \emph{Journal of the American Statistical
+#' Association}, 112, 721-732. <DOI:10.1080/01621459.2016.1165103>}
+#' @export
+#' @examples
+#' 
+#' pd1 <- ewens(mass(1),50)
+#' 
+#' decay <- decay.exponential(temperature(1.0),dist(scale(USArrests)))
+#' attraction <- attraction(permutation(n.items=50,fixed=FALSE), decay)
+#' pd2 <- ewens.pitman.attraction(mass(1), discount(0.05), attraction)
+#' 
+#' pd3 <- ddcrp(mass(1), attraction)
+#' 
 ewens <- function(mass, n.items, names=paste0("c",1:n.items)) {
   n.items <- as.integer(n.items[1])
   if ( n.items < 0 ) stop("'n.items' must be nonnegative.")
+  if ( ! inherits(mass,"shallot.mass") ) stop("'mass' must be a result from the 'mass' function.")
   result <- list(mass=mass,n.items=n.items,names=names)
   class(result) <- "shallot.distribution.ewens"
   result
 }
 
+#' @rdname ewens
+#' @export
 print.shallot.distribution.ewens <- function(x, ...) {
   cat("Ewens distribution with\n")
   cat("  ",x$n.items," items\n",sep="")
@@ -391,15 +620,19 @@ print.shallot.distribution.ewens <- function(x, ...) {
 
 
 
-# Ewens Pitman
+#' @rdname ewens
+#' @export
 ewens.pitman <- function(mass, discount, n.items, names=paste0("c",1:n.items)) {
   n.items <- as.integer(n.items[1])
   if ( n.items < 0 ) stop("'n.items' must be nonnegative.")
+  if ( ! inherits(mass,"shallot.mass") ) stop("'mass' must be a result from the 'mass' function.")
   result <- list(mass=mass,discount=discount,n.items=n.items,names=names)
   class(result) <- "shallot.distribution.ewensPitman"
   result
 }
 
+#' @rdname ewens
+#' @export
 print.shallot.distribution.ewensPitman <- function(x, ...) {
   cat("Ewens Pitman distribution with\n")
   cat("  ",x$n.items," items\n",sep="")
@@ -413,13 +646,17 @@ print.shallot.distribution.ewensPitman <- function(x, ...) {
 
 
 
-# Ewens Attraction
+#' @rdname ewens
+#' @export
 ewens.attraction <- function(mass, attraction) {
+  if ( ! inherits(mass,"shallot.mass") ) stop("'mass' must be a result from the 'mass' function.")
   result <- list(mass=mass,attraction=attraction,n.items=attraction$n.items,names=attraction$names)
   class(result) <- "shallot.distribution.ewensAttraction"
   result
 }
 
+#' @rdname ewens
+#' @export
 print.shallot.distribution.ewensAttraction <- function(x, ...) {
   cat("Ewens Attraction distribution with\n")
   cat("  ")
@@ -432,13 +669,17 @@ print.shallot.distribution.ewensAttraction <- function(x, ...) {
 
 
 
-# Ewens Pitman Attraction
+#' @rdname ewens
+#' @export
 ewens.pitman.attraction <- function(mass, discount, attraction) {
+  if ( ! inherits(mass,"shallot.mass") ) stop("'mass' must be a result from the 'mass' function.")
   result <- list(mass=mass,discount=discount,attraction=attraction,n.items=attraction$n.items,names=attraction$names)
   class(result) <- "shallot.distribution.ewensPitmanAttraction"
   result
 }
 
+#' @rdname ewens
+#' @export
 print.shallot.distribution.ewensPitmanAttraction <- function(x, ...) {
   cat("Ewens Pitman Attraction distribution with\n")
   cat("  ")
@@ -453,13 +694,17 @@ print.shallot.distribution.ewensPitmanAttraction <- function(x, ...) {
 
 
 
-# Distance dependent Chinese restaurant process (ddCRP)
+#' @rdname ewens
+#' @export
 ddcrp <- function(mass, attraction) {
+  if ( ! inherits(mass,"shallot.mass") ) stop("'mass' must be a result from the 'mass' function.")
   result <- list(mass=mass,attraction=attraction,n.items=attraction$n.items,names=attraction$names)
   class(result) <- "shallot.distribution.ddcrp"
   result
 }
 
+#' @rdname ewens
+#' @export
 print.shallot.distribution.ddcrp <- function(x, ...) {
   cat("ddCRP distribution with\n")
   cat("  ")
@@ -471,8 +716,56 @@ print.shallot.distribution.ddcrp <- function(x, ...) {
 # ddcrp(mass(2.3),a)
 
 
-
-# Distribution of the number of subsets
+#' Number of Subsets
+#' 
+#' These functions either sample the number of subsets for supported partition
+#' distributions or computes probabilities, means, and variances of these
+#' distributions.
+#' 
+#' 
+#' @aliases nsubsets.random nsubsets.probability nsubsets.average
+#' nsubsets.variance
+#' @param x An object of class \code{shallot.distribution}.
+#' @param n.samples An integer containing the number of samples.
+#' @param n.subsets An integer containing the number of subsets.
+#' @param ... Currently ignored.
+#' @return The \code{\link{nsubsets.random}} function returns a vector of
+#' random samples of the number of subsets in the distribution \var{x}.
+#' 
+#' The \code{\link{nsubsets.probability}} function returns the probability that
+#' the number of subsets is \var{n.subsets} in the distribution \var{x}.
+#' Depending on the number of items and the value of \var{n.subsets}, this
+#' function can be computationally intensive.
+#' 
+#' The \code{\link{nsubsets.average}} and \code{\link{nsubsets.variance}}
+#' functions return the mean and variances, respectively, of the number of
+#' subsets in the distribution \var{x}.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @seealso \code{\link{partition.distribution}}
+#' @references \href{https://doi.org/10.1080/01621459.2016.1165103}{Dahl, D. B.,
+#' Day, R., and Tsai, J. (2017), Random Partition Distribution Indexed by
+#' Pairwise Information, \emph{Journal of the American Statistical
+#' Association}, 112, 721-732. <DOI:10.1080/01621459.2016.1165103>}
+#' @export
+#' @examples
+#' 
+#' \donttest{
+#' pd <- ewens.pitman.attraction(
+#'         mass(1),
+#'         discount(0.05),
+#'         attraction(permutation(n.items=50,fixed=FALSE),
+#'           decay.exponential(temperature(1.0),dist(scale(USArrests)))))
+#' mean(nsubsets.random(pd,1000))
+#' nsubsets.average(pd)
+#' 
+#' pde <- ewens(mass(1),50)
+#' nsubsets.variance(pde)
+#' nsubsets.probability(pde,4)
+#' \dontshow{
+#' rscala::scalaDisconnect(shallot:::s)
+#' }
+#' }
+#'
 nsubsets.random <- function(x,n.samples) {
   n.samples <- as.integer(n.samples)
   if ( n.samples < 0 ) stop("'n.samples' must be positive.")
@@ -497,7 +790,8 @@ nsubsets.random <- function(x,n.samples) {
 
 
 
-# Probability of the number of subsets
+#' @rdname nsubsets.random
+#' @export
 nsubsets.probability <- function(x,n.subsets) {
   n.subsets <- as.integer(n.subsets)
   if ( any( ! ( c("mass","n.items") %in% names(x) ) ) ) stop("Unrecognized distribution.")
@@ -522,8 +816,8 @@ nsubsets.probability <- function(x,n.subsets) {
 # nsubsets.probability(ewens.pitman.attraction(mass(n.items=a$n.items,fixed=TRUE),discount(0.1,fixed=TRUE),a),4)
 
 
-
-# Expected number of subsets
+#' @rdname nsubsets.random
+#' @export
 nsubsets.average <- function(x) {
   if ( any( ! ( c("mass","n.items") %in% names(x) ) ) ) stop("Unrecognized distribution.")
   if ( ! x$mass$fixed ) stop("'mass' must be fixed for this function, but emperical estimates are available through the 'nsubsets.random' function.")
@@ -548,7 +842,8 @@ nsubsets.average <- function(x) {
 
 
 
-# Variance of the number of subsets
+#' @rdname nsubsets.random
+#' @export
 nsubsets.variance <- function(x) {
   if ( any( ! ( c("mass","n.items") %in% names(x) ) ) ) stop("Unrecognized distribution.")
   if ( ! x$mass$fixed ) stop("'mass' must be fixed for this function, but emperical estimates are available through the 'nsubsets.random' function.")
@@ -646,7 +941,37 @@ nsubsets.variance <- function(x) {
   '
 }
 
-# Sample partitions.
+#' Sample Partitions from Partition Distributions
+#' 
+#' This function samples partitions from the Ewens, Ewens-Pitman, Ewens
+#' attraction, Ewens-Pitman attraction, and ddCRP distributions.
+#' 
+#' 
+#' @aliases sample.partitions print.shallot.samples.raw
+#' @param x An object of class \code{shallot.distribution} obtained, for
+#' example, from the \code{\link{ewens.pitman.attraction}} function.
+#' @param n.draws An integer representing the desired number of samples.  Due
+#' to parallelization, slightly more samples may be returned.
+#' @param parallel Should sampling be done in parallel by simultaneously using
+#' all CPU cores?
+#' @param ... Currently ignored.
+#' @return An object of class \code{shallot.samples.raw} which can be
+#' subsequently be used in \code{\link{process.samples}},
+#' \code{\link{pairwise.probabilities}}, \code{\link{estimate.partition}},
+#' @note If this function is interrupted by the user, the computation engine
+#' will be broken and subsequent calls to package functions may fail until a
+#' new session is started.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @seealso \code{\link{partition.distribution}},
+#' \code{\link{process.samples}}, \code{\link{pairwise.probabilities}},
+#' \code{\link{estimate.partition}} \code{\link{sample.partitions.posterior}}
+#' @examples
+#' 
+#' \dontrun{
+#' example(shallot)
+#' }
+#' 
+#' @export sample.partitions
 sample.partitions <- function(x, n.draws, parallel=TRUE) {
   forwardSampler <- if ( inherits(x,"shallot.distribution.ewens") ) {
     .sample.ewens(x$n.items,.massFactory(x$mass))
@@ -656,22 +981,143 @@ sample.partitions <- function(x, n.draws, parallel=TRUE) {
     .sample.ewensAttraction(x$n.items,.massFactory(x$mass),.attractionFactory(x$attraction))
   } else if ( inherits(x,"shallot.distribution.ewensPitmanAttraction") ) {
     .sample.ewensPitmanAttraction(x$n.items,.massFactory(x$mass),.discountFactory(x$discount),.attractionFactory(x$attraction))
+#  } else if ( inherits(x,"shallot.distribution.DDCRP") ) {
+#    .sample.DDCRP(x$n.items,.massFactory(x$mass),.discountFactory(x$discount),.attractionFactory(x$attraction))   
   } else stop("Unrecognized partition distribution.")
   ref <- .sampleForward(n.draws,s$rdg(),forwardSampler,parallel)
   structure(list(ref=ref, names=x$names), class="shallot.samples.raw")
 }
 
+#' @export sample.partitions
+#' @export
 print.shallot.samples.raw <- function(x, ...) {
   cat("raw partition samples --- use the 'process.samples' function to extract information\n")
 }
 
+#' @export sample.partitions
+#' @export
 print.shallot.samples.full <- function(x, ...) {
   cat("raw partition samples with hyperparameter values --- use the 'process.samples' function to extract information\n")
 }
 
 
 
-# Posterior simulation via MCMC.
+#' Sample Partitions from Posterior Distribution of Partition
+#' 
+#' This function samples partitions from the posterior distribution of a
+#' partition based on a user-supplied likelihood and the following prior
+#' partition distributions: Ewens, Ewens-Pitman, Ewens attraction, Ewens-Pitman
+#' attraction, and ddCRP distributions.
+#' 
+#' 
+#' @param partition An object of class \code{shallot.distribution.data}
+#' @param sampling.model An object of class \code{shallot.distribution.data}
+#' obtained from the \code{\link{sampling.model}} function.
+#' @param partition.model An object of class \code{shallot.distribution}
+#' obtained, for example, from the \code{\link{ewens.pitman.attraction}}
+#' function.
+#' @param n.draws An integer representing the desired number of samples.
+#' @param massRWSD The standard deviation of the random walk proposal for
+#' updating the mass parameter.
+#' @param discountRWSD The standard deviation of the random walk proposal for
+#' updating the discount parameter.
+#' @param k The number of items to shuffle when proposing an update for the
+#' permutation.
+#' @param temperatureRWSD The standard deviation of the random walk proposal
+#' for updating the temperature parameter.
+#' @param progress.bar Should a progress bar be shown while sampling?
+#' @return An object of class \code{shallot.samples.raw} which can be
+#' subsequently be used in \code{\link{process.samples}},
+#' \code{\link{pairwise.probabilities}}, \code{\link{estimate.partition}},
+#' @note If this function is interrupted by the user, the computation engine
+#' will be broken and subsequent calls to package functions may fail until a
+#' new session is started.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @importFrom utils capture.output txtProgressBar
+#' @seealso \code{\link{partition.distribution}},
+#' \code{\link{process.samples}}, \code{\link{pairwise.probabilities}},
+#' \code{\link{estimate.partition}} \code{\link{sample.partitions}}
+#' @examples
+#' 
+#' 1+2
+#' #\donttest{
+#' #mass <- mass(1.0, fixed=TRUE)
+#' #discount <- discount(0.05, fixed=TRUE)
+#' #distance <- dist(scale(USArrests[1:9,]))
+#' #if ( min(distance[upper.tri(distance)],na.rm=TRUE) == 0 )
+#' #  stop("Oops, distances must be strictly positive.")
+#' #
+#' #n.items <- attr(distance,"Size")
+#' #permutation <- permutation(n.items=n.items, fixed=FALSE)
+#' #temperature <- temperature(2,fixed=TRUE)
+#' #attraction <- attraction(permutation,decay.exponential(temperature,distance))
+#' #partition.distribution <- ewens.pitman.attraction(mass, discount, attraction)
+#' #
+#' ### Model inputs.
+#' #data <- c(-1.48, -1.40, -1.16, -1.08, -1.02, 0.14, 0.51, 0.53, 0.78)
+#' #sigma  <- 0.1
+#' #mu0    <- 0.0
+#' #sigma0 <- 1.0
+#' #
+#' ### Derived values.
+#' #s2 <- sigma * sigma
+#' #s02 <- sigma0 * sigma0
+#' #s02Inv <- 1.0 / s02
+#' #c <- -1.0 / (2.0 * s2)
+#' #
+#' ### Sampling model of Neal (JCGS, 2009)
+#' ### Function to perform an MCMC update of the parameter.
+#' #sample.parameter <- function(indices=scalaType("D1"), parameter=scalaType("D0")) {
+#' #  sum <- sum(data[indices])
+#' #  variance <- 1 / (s02Inv + length(indices) / s2)
+#' #  mean <- variance * (mu0 / s02 + sum / s2)
+#' #  rnorm(1, mean, sqrt(variance))
+#' #}
+#' #
+#' #library(rscala)
+#' #s <- shallot:::s
+#' #sample.parameter.compiled <- s(data=data,mu0=mu0,s2=s2,s02=s02,s02Inv=s02Inv) ^ sample.parameter
+#' #
+#' #
+#' ### Function to evaluate the likelihood contribution for an observation.
+#' #log.density <- function(i=scalaType("D0"), indices=scalaType("D1"), parameter=scalaType("D0")) {
+#' #  resid <- data[i] - parameter
+#' #  c * resid * resid
+#' #}
+#' #
+#' #log.density.compiled <- s(data=data,c=c) ^ log.density
+#' #
+#' #sampling.model <- sampling.model(sample.parameter, log.density)
+#' #
+#' ### Perform posterior sampling.
+#' #initial.partition <- rep(1,length(data))
+#' #n.draws <- 10
+#' #raw <- sample.partitions.posterior(initial.partition,sampling.model,partition.distribution,
+#' #                                   massRWSD=3,temperatureRWSD=1,n.draws)
+#' #samples.format1 <- process.samples(raw)
+#' #parameterMatrix <- t(sapply(seq_len(n.draws), function(i) {
+#' #  unlist(samples.format1$parameters[[i]])[samples.format1$labels[i,]]
+#' #}))
+#' #
+#' #tail(samples.format1$hyperparameters)
+#' #
+#' ### Shrinkage to group means?
+#' #plot(data,apply(parameterMatrix,2,mean))
+#' #abline(a=0,b=1)
+#' #
+#' #samples.format1$hyperparameters
+#' #
+#' ### Post processing to find the partition estimate.
+#' #library(sdols)
+#' #pp <- expectedPairwiseAllocationMatrix(samples.format1$labels)
+#' #est <- salso(pp)
+#' #plot(confidence(est,pp))
+#' #\dontshow{
+#' #rscala::scalaDisconnect(shallot:::s)
+#' #}
+#' #}
+#' 
+#' @export sample.partitions.posterior
 sample.partitions.posterior <- function(partition, sampling.model, partition.model, n.draws, massRWSD=0.5, discountRWSD=0.1, k=min(length(partition),25), temperatureRWSD=0.5, progress.bar=interactive()) {
   pmR <- partition.model
   sampler <- function(p=s$".null_Partition[org.ddahl.rscala.RObject]"(),
@@ -820,7 +1266,25 @@ sample.partitions.posterior <- function(partition, sampling.model, partition.mod
 
 
 
-# Make probability mass function.
+#' Obtain the Probability Mass Function of a Partition Distribution
+#' 
+#' This function returns the probability mass function (pmf) of a partition
+#' distribution.
+#' 
+#' 
+#' @param x An object of class \code{shallot.distribution} obtained, for
+#' example, from the \code{\link{ewens.pitman.attraction}} function.
+#' @return A function that takes a partition (as a vector in cluster label
+#' notation) and returns the probability --- or, if \code{log=TRUE}, the log of
+#' the probability --- of the supplied partition.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @examples
+#' 
+#' \dontrun{
+#' example(shallot)
+#' }
+#' 
+#' @export partition.pmf
 partition.pmf <- function(x) {
   distribution <- .partitionModel(x)
   pmf <- distribution$logProbability
@@ -836,11 +1300,59 @@ partition.pmf <- function(x) {
 
 
 
-# Sampling model
-sampling.model <- function(sample.parameter, log.density) {
-  
-}
-
+#' Specifying a Sampling Model
+#' 
+#' These functions set the mass, discount, and temperature parameters and, in
+#' the case of them being random, specify the parameters of their distribution.
+#' 
+#' 
+#' @param sample.parameter A function taking two arguments with names
+#' \code{indices} and \code{parameter} having default values \code{c()} and
+#' \code{NULL}, respectively.  With those default values, the function should
+#' return a sample from the centering distribution as an object of any type.
+#' Otherwise, \code{indices} is a vector of integers indicating the cluster
+#' elements and \code{parameter} is the current value of the parameter for the
+#' cluster, and the function should return an updated value for the cluster
+#' parameter based on a valid MCMC update.
+#' @param log.density A function taking three arguments with names \code{i},
+#' \code{indices}, and \code{parameter} and returning a double giving the log
+#' of the likelihood contribution of item \code{i} to a clustering with members
+#' \code{indices} and parameter \code{parameter}.
+#' @return An object of class \code{shallot.distribution.data}.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @examples
+#' 
+#' ## Model inputs.
+#' data <- c(-1.48, -1.40, -1.16, -1.08, -1.02, 0.14, 0.51, 0.53, 0.78)
+#' sigma  <- 0.1
+#' mu0    <- 0.0
+#' sigma0 <- 1.0
+#' 
+#' ## Derived values.
+#' s2 <- sigma * sigma
+#' s02 <- sigma0 * sigma0
+#' s02Inv <- 1.0 / s02
+#' c <- -1.0 / (2.0 * s2)
+#' 
+#' ## Sampling model of Neal (JCGS, 2009)
+#' ## Function to perform an MCMC update of the parameter.
+#' sample.parameter <- function(indices=c(), parameter=NULL) {
+#'   sum <- sum(data[indices])
+#'   variance <- 1 / (s02Inv + length(indices) / s2)
+#'   mean <- variance * (mu0 / s02 + sum / s2)
+#'   rnorm(1, mean=mean, sd=sqrt(variance))
+#' }
+#' 
+#' ## Function to evaluate the likelihood contribution for an observation.
+#' log.density <- function(i, indices, parameter) {
+#'   resid <- data[i] - parameter
+#'   c * resid * resid
+#' }
+#' 
+#' sm <- sampling.model(sample.parameter, log.density)
+#' sm
+#' 
+#' @export sampling.model
 sampling.model <- function(sample.parameter, log.density) {
   if ( ! is.function(sample.parameter) ) stop("'sample.parameter' should be a function.")
   if ( length(formals(sample.parameter)) != 2 ) stop("'sample.parameter' should take two arguments named 'indices' and 'parameter'.")
@@ -882,7 +1394,27 @@ sampling.model <- function(sample.parameter, log.density) {
 
 
 
-# Process partitions that were sampled.
+#' Process Sampled Partitions
+#' 
+#' This function extracts the partitions from the results of the
+#' \code{\link{sample.partitions} function.}
+#' 
+#' This function extracts the sampled partitions from the results of the
+#' \code{\link{sample.partitions} function.}
+#' 
+#' @param x An object from the \code{\link{sample.partitions}} function.
+#' @return A list containing a matrix of cluster labels in which each row
+#' represents a clusterings.  The list also contains sampled model parameters
+#' if \code{sample.parameter} is not \code{NULL}.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @seealso \code{\link{sample.partitions}}
+#' @examples
+#' 
+#' \dontrun{
+#' example(shallot)
+#' }
+#' 
+#' @export process.samples
 process.samples <- function(x) {
   if ( ( ! inherits(x,"shallot.samples.raw") ) && ( ! inherits(x,"shallot.samples.full") ) ) stop("'x' should be a result from the functions 'sample.partitions' or 'sample.partitions.posterior'.")
   if ( inherits(x,"shallot.samples.full") ) {
@@ -901,7 +1433,26 @@ process.samples <- function(x) {
 
 
 
-# Pairwise Probabilities
+#' Enumerate Partitions
+#' 
+#' This function enumerates all possible partitions for a given number of
+#' items.
+#' 
+#' This function returns an enumeration of the partition of \code{n.items}
+#' items.
+#' 
+#' @param n.items An integer given then number of items to partition.
+#' @return A matrix of cluster labels in which each row represents a
+#' clusterings.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @seealso \code{\link{process.samples}}
+#' @examples
+#' 
+#' \dontrun{
+#' example(shallot)
+#' }
+#' 
+#' @export enumerate.partitions
 enumerate.partitions <- function(n.items) {
   ref <- s$.Partition.enumerate(s ^ function() {}, as.integer(n.items)[1])
   .partitionsToMatrix(ref)
@@ -909,7 +1460,42 @@ enumerate.partitions <- function(n.items) {
 
 
 
-# Pairwise Probabilities
+#' Pairwise Probabilities
+#' 
+#' These functions relate to the pairwise probabilities that two items are
+#' clustered together, i.e., belong to the same subset in a partition.
+#' 
+#' \code{\link{pairwise.probabilities}} calculates the pairwise probabilities
+#' that two items are clustered together, i.e., belong to the same subset in a
+#' partition.
+#' 
+#' \code{\link[=as.matrix.shallot.pairwiseProbability]{as.matrix}} converts the
+#' results of \code{\link{pairwise.probabilities}} to an matrix.
+#' 
+#' @aliases pairwise.probabilities print.shallot.pairwiseProbability
+#' as.matrix.shallot.pairwiseProbability
+#' @param x An object of class \code{shallot.samples.raw} when supplied to
+#' \code{\link{pairwise.probabilities}} or an object of class
+#' \code{shallot.pairwiseProbability} when supplied to \code{\link{print}} and
+#' \code{\link{as.matrix}}.
+#' @param parallel Should all of the CPU cores should be used?  Defaults to
+#' \code{TRUE}.
+#' @param ... Currently ignored.
+#' @return The \code{\link{pairwise.probabilities}} function returns an object
+#' of class \code{shallot.pairwiseProbability}.
+#' 
+#' The \code{\link[=as.matrix.shallot.pairwiseProbability]{as.matrix}} function
+#' returns a square matrix.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @seealso \code{\link{sample.partitions}}, \code{\link{process.samples}},
+#' \code{\link{estimate.partition}}
+#' @examples
+#' 
+#' \dontrun{
+#' example(shallot)
+#' }
+#' 
+#' @export pairwise.probabilities
 pairwise.probabilities <- function(x, parallel=TRUE) {
   if ( inherits(x,"shallot.samples.full") ) x <- x$raw
   if ( ! inherits(x,"shallot.samples.raw") ) stop("'x' should be a result from the functions 'sample.partitions' or 'sample.partitions.posterior'.")
@@ -919,17 +1505,58 @@ pairwise.probabilities <- function(x, parallel=TRUE) {
   structure(result, class="shallot.pairwiseProbability")
 }
 
+#' @export pairwise.probabilities
+#' @export
 print.shallot.pairwiseProbability <- function(x, ...) {
   cat("pairwise probabilities for ",x$n.items," items --- use 'as.matrix' function to obtain matrix\n",sep="")
 }
 
+#' @export pairwise.probabilities
+#' @export
 as.matrix.shallot.pairwiseProbability <- function(x, ...) {
   structure(x$ref$toArray(), dimnames=list(x$names,x$names))
 }
 
 
 
-# Estimate partition
+#' Estimate Partition
+#' 
+#' This function returns a partition that summarizes the partition distribution
+#' using the least-square clustering method (Dahl 2006), with extensions to
+#' perform greedy optimization and limit the number of subsets.
+#' 
+#' 
+#' @param x An object from the \code{\link{sample.partitions}} function.
+#' @param pairwise.probabilities An object of class
+#' \code{shallot.pairwiseProbability} obtained from
+#' \code{\link{pairwise.probabilities}}.  If not supplied, it will be computed
+#' from \var{x}.
+#' @param max.subsets An integer limiting the number of subsets.  Defaults to
+#' \code{0}, which does not impose a constraint on the number of subsets.
+#' @param max.scans An integer controlling the greedy search.  Defaults to
+#' \code{0}, which disables the greedy search.
+#' @param parallel Should all of the CPU cores should be used?  Defaults to
+#' \code{TRUE}.
+#' @return A partition as a vector of cluster labels.
+#' @author David B. Dahl \email{dahl@@stat.byu.edu}
+#' @seealso \code{\link{sample.partitions}}, \code{\link{process.samples}},
+#' \code{\link{plot.partition}}, \code{\link{adj.rand.index}}
+#' @references \href{https://doi.org/10.1080/01621459.2016.1165103}{Dahl, D. B.,
+#' Day, R., and Tsai, J. (2017), Random Partition Distribution Indexed by
+#' Pairwise Information, \emph{Journal of the American Statistical
+#' Association}, 112, 721-732. <DOI:10.1080/01621459.2016.1165103>}
+#' 
+#' Dahl, D. B. (2006), Model-Based Clustering for Expression Data via a
+#' Dirichlet Process Mixture Model, in \emph{Bayesian Inference for Gene
+#' Expression and Proteomics}, Kim-Anh Do, Peter Mueller, Marina Vannucci
+#' (Eds.), Cambridge University Press.
+#' @examples
+#' 
+#' \dontrun{
+#' example(shallot)
+#' }
+#' 
+#' @export estimate.partition
 estimate.partition <- function(x, pairwise.probabilities=NULL, max.subsets=0, max.scans=0, parallel=TRUE) {
   if ( inherits(x,"shallot.samples.full") ) x <- x$raw
   if ( ! inherits(x,"shallot.samples.raw") ) stop("'x' should be a result from the functions 'sample.partitions' or 'sample.partitions.posterior'.")
@@ -950,5 +1577,3 @@ estimate.partition <- function(x, pairwise.probabilities=NULL, max.subsets=0, ma
 .partition2partition <- function(partition) {
   stop("Not yet implemented.")
 }
-
-
